import { useEffect, useState } from 'react';
import { deleteBarcode, getAllBarcodes, updateBarcodeTags } from '../db';

function HistoryPage() {
  const [barcodes, setBarcodes] = useState([]);
  const [loading, setLoading] = useState(true);
  const [tagInputs, setTagInputs] = useState({});
  const [allTags, setAllTags] = useState([]);
  const [selectedTag, setSelectedTag] = useState('');
  const [selectedIds, setSelectedIds] = useState(new Set());

  const fetchBarcodes = async () => {
    try {
      const allBarcodes = await getAllBarcodes();
      const sortedBarcodes = allBarcodes.sort((a, b) => b.createdAt - a.createdAt);
      setBarcodes(sortedBarcodes);

      const uniqueTags = [...new Set(allBarcodes.flatMap(b => b.tags || []))];
      setAllTags(uniqueTags);

      const initialTags = {};
      sortedBarcodes.forEach(b => {
        initialTags[b.id] = (b.tags || []).join(', ');
      });
      setTagInputs(initialTags);
    } catch (error) {
      console.error('Failed to fetch barcodes:', error);
    } finally {
      setLoading(false);
    }
  };

  useEffect(() => {
    fetchBarcodes();
  }, []);

  const handleDelete = async (id) => {
    try {
      await deleteBarcode(id);
      fetchBarcodes(); // Refresh list
    } catch (error) {
      console.error('Failed to delete barcode:', error);
    }
  };

  const handleTagsChange = (id, value) => {
    setTagInputs(prev => ({ ...prev, [id]: value }));
  };

  const handleUpdateTags = async (id) => {
    const tags = tagInputs[id].split(',').map(t => t.trim()).filter(Boolean);
    try {
      await updateBarcodeTags(id, tags);
      await fetchBarcodes();
    } catch (error) {
      console.error('Failed to update tags:', error);
    }
  };

  const handleSelect = (id) => {
    setSelectedIds(prev => {
      const newSet = new Set(prev);
      if (newSet.has(id)) {
        newSet.delete(id);
      } else {
        newSet.add(id);
      }
      return newSet;
    });
  };

  const handleBulkDelete = async () => {
    try {
      for (const id of selectedIds) {
        await deleteBarcode(id);
      }
      setSelectedIds(new Set());
      await fetchBarcodes();
    } catch (error) {
      console.error('Failed to bulk delete barcodes:', error);
    }
  };

  const handleBulkExport = () => {
    const dataToExport = barcodes.filter(b => selectedIds.has(b.id));
    if (dataToExport.length === 0) return;

    const csvContent = "data:text/csv;charset=utf-8,"
      + "Code,Tags,ScannedAt\n"
      + dataToExport.map(b =>
        `"${b.code}","${(b.tags || []).join(',')}","${new Date(b.createdAt).toISOString()}"`
      ).join("\n");

    const encodedUri = encodeURI(csvContent);
    const link = document.createElement("a");
    link.setAttribute("href", encodedUri);
    link.setAttribute("download", "barcodes.csv");
    document.body.appendChild(link);
    link.click();
    document.body.removeChild(link);
  };

  if (loading) {
    return <div className="text-center mt-8">正在加载...</div>;
  }

  const filteredBarcodes = selectedTag
    ? barcodes.filter(b => (b.tags || []).includes(selectedTag))
    : barcodes;

  return (
    <div className="px-4 pt-4 pb-20">
      <h1 className="text-2xl font-bold mb-4">历史记录</h1>

      <div className="mb-4 space-y-4">
        <div>
          <label htmlFor="tag-filter" className="block text-sm font-medium text-gray-700 dark:text-gray-300">按标签筛选:</label>
          <select
            id="tag-filter"
            value={selectedTag}
            onChange={(e) => setSelectedTag(e.target.value)}
            className="mt-1 block w-full pl-3 pr-10 py-2 text-base border-gray-300 dark:border-gray-600 bg-white dark:bg-gray-700 text-gray-900 dark:text-white focus:outline-none focus:ring-indigo-500 focus:border-indigo-500 sm:text-sm rounded-md"
          >
            <option value="">所有</option>
            {allTags.map(tag => (
              <option key={tag} value={tag}>{tag}</option>
            ))}
          </select>
        </div>
        {selectedIds.size > 0 && (
          <div className="flex space-x-2">
            <button onClick={handleBulkDelete} className="bg-red-600 hover:bg-red-700 text-white font-bold py-2 px-4 rounded">
              删除选中 ({selectedIds.size})
            </button>
            <button onClick={handleBulkExport} className="bg-green-600 hover:bg-green-700 text-white font-bold py-2 px-4 rounded">
              导出选中 ({selectedIds.size})
            </button>
          </div>
        )}
      </div>

      {filteredBarcodes.length === 0 ? (
        <p>没有符合条件的记录。</p>
      ) : (
        <ul className="space-y-3">
          {filteredBarcodes.map((barcode) => (
            <li
              key={barcode.id}
              className={`bg-white dark:bg-gray-800 shadow-md rounded-lg p-4 ${selectedIds.has(barcode.id) ? 'ring-2 ring-blue-500' : ''}`}
            >
              <div className="flex justify-between items-start">
                <div className="flex items-start space-x-3">
                  <input
                    type="checkbox"
                    checked={selectedIds.has(barcode.id)}
                    onChange={() => handleSelect(barcode.id)}
                    className="mt-1 h-5 w-5 text-blue-600 border-gray-300 dark:border-gray-600 rounded bg-gray-100 dark:bg-gray-700"
                  />
                  <div>
                    <p className="font-mono text-lg text-gray-800 dark:text-gray-200">{barcode.code}</p>
                    <p className="text-sm text-gray-500 dark:text-gray-400">
                      {new Date(barcode.createdAt).toLocaleString()}
                    </p>
                  </div>
                </div>
                <button
                  onClick={() => handleDelete(barcode.id)}
                  className="bg-red-500 hover:bg-red-600 text-white font-bold py-1 px-3 rounded flex-shrink-0"
                >
                  删除
                </button>
              </div>
              <div className="mt-3">
                <div className="flex items-center space-x-2">
                  <input
                    type="text"
                    value={tagInputs[barcode.id] || ''}
                    onChange={(e) => handleTagsChange(barcode.id, e.target.value)}
                    placeholder="添加标签 (用逗号分隔)"
                    className="border rounded px-2 py-1 w-full text-sm bg-gray-50 dark:bg-gray-700 border-gray-300 dark:border-gray-600 text-gray-900 dark:text-white"
                  />
                  <button
                    onClick={() => handleUpdateTags(barcode.id)}
                    className="bg-blue-500 hover:bg-blue-600 text-white font-bold py-1 px-3 rounded text-sm"
                  >
                    更新
                  </button>
                </div>
                {barcode.tags && barcode.tags.length > 0 && (
                  <div className="mt-2 flex flex-wrap gap-1">
                    {barcode.tags.map(tag => (
                      <span key={tag} className="bg-gray-200 dark:bg-gray-600 text-gray-700 dark:text-gray-200 text-xs font-semibold px-2 py-1 rounded-full">
                        {tag}
                      </span>
                    ))}
                  </div>
                )}
              </div>
            </li>
          ))}
        </ul>
      )}
    </div>
  );
}

export default HistoryPage;
