import Quagga from '@ericblade/quagga2';
import { useEffect, useRef, useState } from 'react';
import { addBarcode } from '../db';

function LiveScannerPage() {
  const videoRef = useRef(null);
  const [error, setError] = useState(null);
  const scannerRef = useRef(false);
  const [scannedResults, setScannedResults] = useState([]);
  const [selectedResult, setSelectedResult] = useState(null);
  const [showSuccess, setShowSuccess] = useState(false);
  const [strictMode, setStrictMode] = useState(false);
  const [selectedFormat, setSelectedFormat] = useState('all');
  const [videoDevices, setVideoDevices] = useState([]);
  const [selectedDeviceId, setSelectedDeviceId] = useState('');

  const handleSave = async () => {
    if (selectedResult) {
      try {
        await addBarcode(selectedResult.code);
        setShowSuccess(true);
        setScannedResults([]);
        setSelectedResult(null);
        setTimeout(() => setShowSuccess(false), 2000);
      } catch (err) {
        console.error("Failed to save barcode:", err);
        setError("保存失败，请重试。");
      }
    }
  };

  useEffect(() => {
    const getVideoDevices = async () => {
      try {
        const devices = await navigator.mediaDevices.enumerateDevices();
        const videoInputs = devices.filter(
          (device) => device.kind === 'videoinput'
        );
        setVideoDevices(videoInputs);
      } catch (err) {
        setError('无法获取摄像头列表。');
        console.error('Could not get video devices:', err);
      }
    };
    getVideoDevices();
  }, []);

  const streamRef = useRef(null);

  useEffect(() => {
    const stopScanner = () => {
      if (scannerRef.current) {
        Quagga.stop();
        scannerRef.current = false;
      }
      if (streamRef.current) {
        streamRef.current.getTracks().forEach(track => track.stop());
        streamRef.current = null;
      }
      if (videoRef.current) {
        videoRef.current.srcObject = null;
      }
    };

    const startScanner = async () => {
      stopScanner();
      setScannedResults([]);
      setSelectedResult(null);
      setError(null);

      try {
        const stream = await navigator.mediaDevices.getUserMedia({
          video: { deviceId: selectedDeviceId },
        });
        streamRef.current = stream;

        if (videoRef.current) {
          videoRef.current.srcObject = streamRef.current;
          videoRef.current.play().catch(e => console.error("Video play failed:", e));
        }

        Quagga.init(
          {
            inputStream: {
              name: 'Live',
              type: 'LiveStream',
              target: videoRef.current,
              constraints: {
                width: 640,
                height: 480,
              },
            },
            decoder: {
              readers: ['code_128_reader', 'ean_reader', 'ean_8_reader', 'code_39_reader', 'code_39_vin_reader', 'codabar_reader', 'upc_reader', 'upc_e_reader', 'i2of5_reader'],
            },
          },
          (err) => {
            if (err) {
              setError('QuaggaJS 初始化失败: ' + err.message);
              console.error(err);
              return;
            }
            Quagga.start();
            scannerRef.current = true;
          }
        );

        Quagga.onProcessed((result) => {
          const drawingCtx = Quagga.canvas.ctx.overlay;
          const drawingCanvas = Quagga.canvas.dom.overlay;

          if (result) {
            if (result.boxes) {
              drawingCtx.clearRect(0, 0, parseInt(drawingCanvas.getAttribute("width")), parseInt(drawingCanvas.getAttribute("height")));
              result.boxes.filter(function (box) {
                return box !== result.box;
              }).forEach(function (box) {
                Quagga.ImageDebug.drawPath(box, { x: 0, y: 1 }, drawingCtx, { color: "green", lineWidth: 2 });
              });
            }

            if (result.box) {
              Quagga.ImageDebug.drawPath(result.box, { x: 0, y: 1 }, drawingCtx, { color: "#00F", lineWidth: 2 });
            }

            if (result.codeResult && result.codeResult.code) {
              Quagga.ImageDebug.drawPath(result.line, { x: 'x', y: 'y' }, drawingCtx, { color: 'red', lineWidth: 3 });
            }
          }
        });

        Quagga.onDetected((data) => {
          const result = data.codeResult;
          if (!result || !result.code) {
            return;
          }

          let maxError = 0;
          if (result.decodedCodes) {
            const errors = result.decodedCodes
              .filter(d => typeof d.error !== 'undefined')
              .map(d => d.error);
            if (errors.length > 0) {
              maxError = Math.max(...errors);
            }
          }

          const newResult = {
            code: result.code,
            format: result.format,
            error: maxError,
          };

          setScannedResults((prevResults) => {
            const existingResultIndex = prevResults.findIndex(
              (r) => r.code === newResult.code
            );

            if (existingResultIndex !== -1) {
              // 如果已存在且新结果的错误率更低，则更新
              if (newResult.error < prevResults[existingResultIndex].error) {
                const updatedResults = [...prevResults];
                updatedResults[existingResultIndex] = newResult;
                return updatedResults;
              }
              // 否则，不更新
              return prevResults;
            } else {
              // 如果不存在，则添加
              return [...prevResults, newResult];
            }
          });
        });

      } catch (err) {
        setError('无法访问摄像头: ' + err.message);
        console.error('Error accessing camera:', err);
      }
    };

    if (selectedDeviceId) {
      startScanner();
    } else {
      stopScanner();
    }

    return () => {
      stopScanner();
    };
  }, [selectedDeviceId]);

  return (
    <div className="relative w-full h-full">
      {/* 视频和 Canvas 容器 */}
      <div id="interactive" className="absolute top-0 left-0 w-full h-full">
        <video
          ref={videoRef}
          autoPlay
          playsInline
          className="w-full h-full object-cover"
        />
        <canvas className="drawingBuffer absolute top-0 left-0 w-full h-full"></canvas>
      </div>

      {/* 浮动 UI 元素 */}
      <div className="absolute top-0 left-0 w-full h-full flex flex-col items-center justify-between p-4 text-gray-800 dark:text-white z-10 pointer-events-none">
        {/* 顶部结果和操作 */}
        <div className="w-full flex flex-col items-center pointer-events-auto">
          <div className="w-full max-w-md bg-white/50 dark:bg-black/50 p-2 rounded-lg mb-4">
            {!selectedDeviceId ? (
              <>
                <label htmlFor="camera-select" className="block mb-1 text-sm font-medium">
                  选择摄像头:
                </label>
                <select
                  id="camera-select"
                  value={selectedDeviceId}
                  onChange={(e) => setSelectedDeviceId(e.target.value)}
                  className="bg-gray-50 border border-gray-300 text-gray-900 text-sm rounded-lg focus:ring-blue-500 focus:border-blue-500 block w-full p-2.5 dark:bg-gray-700 dark:border-gray-600 dark:placeholder-gray-400 dark:text-white"
                >
                  <option value="">-- 请选择 --</option>
                  {videoDevices.map((device, index) => (
                    <option key={device.deviceId} value={device.deviceId}>
                      {device.label || `摄像头 ${index + 1}`}
                    </option>
                  ))}
                </select>
              </>
            ) : (
              <div className="text-center p-2.5">
                当前摄像头: {videoDevices.find(d => d.deviceId === selectedDeviceId)?.label || '已选择'}
              </div>
            )}
          </div>
          {selectedDeviceId && scannedResults.length > 0 && (() => {
            const availableFormats = [...new Set(scannedResults.map(r => r.format))];
            const filteredAndSortedResults = scannedResults
              .filter(result => !strictMode || result.error < 0.2)
              .filter(result => selectedFormat === 'all' || result.format === selectedFormat)
              .sort((a, b) => a.error - b.error);

            return (
              <div className="w-full max-w-md bg-white/50 dark:bg-black/50 p-4 rounded-lg">
                <div className="flex justify-between items-center mb-2">
                  <h2 className="text-lg font-bold">扫描结果:</h2>
                  <div className="flex items-center space-x-4">
                    <label className="flex items-center cursor-pointer">
                      <input
                        type="checkbox"
                        checked={strictMode}
                        onChange={() => setStrictMode(!strictMode)}
                        className="form-checkbox h-5 w-5 text-blue-600"
                      />
                      <span className="ml-2 text-sm font-medium">严格模式</span>
                    </label>
                    <select
                      value={selectedFormat}
                      onChange={(e) => setSelectedFormat(e.target.value)}
                      className="bg-gray-50 border border-gray-300 text-gray-900 text-sm rounded-lg focus:ring-blue-500 focus:border-blue-500 block p-1.5 dark:bg-gray-700 dark:border-gray-600 dark:text-white"
                    >
                      <option value="all">所有格式</option>
                      {availableFormats.map(format => (
                        <option key={format} value={format}>{format}</option>
                      ))}
                    </select>
                  </div>
                </div>
                <div className="space-y-2 max-h-48 overflow-y-auto">
                  {filteredAndSortedResults.map((result) => (
                      <label key={result.code} className="flex items-start space-x-3 cursor-pointer p-2 rounded-md hover:bg-gray-200 dark:hover:bg-gray-700">
                        <input
                          type="radio"
                          name="scannedResult"
                          value={result.code}
                          checked={selectedResult?.code === result.code}
                          onChange={() => setSelectedResult(result)}
                          className="form-radio h-5 w-5 text-blue-600 mt-1"
                        />
                        <div className="flex-1">
                          <span className="font-mono text-base block">{result.code}</span>
                          <div className="text-xs text-gray-600 dark:text-gray-400">
                            <span>格式: {result.format}</span>
                            <span className="ml-2">|</span>
                            <span className="ml-2">错误率: {(result.error * 100).toFixed(2)}%</span>
                          </div>
                        </div>
                      </label>
                    ))}
                </div>
                <div className="mt-4 flex space-x-2">
                  <button
                    onClick={handleSave}
                    disabled={!selectedResult}
                    className="w-full bg-blue-500 hover:bg-blue-600 text-white font-bold py-2 px-4 rounded disabled:bg-gray-500"
                  >
                    保存选中结果
                  </button>
                  <button
                    onClick={() => {
                      setScannedResults([]);
                      setSelectedResult(null);
                    }}
                    className="w-full bg-gray-500 hover:bg-gray-600 text-white font-bold py-2 px-4 rounded"
                  >
                    清空
                  </button>
                </div>
              </div>
            );
          })()}
          {showSuccess && (
            <div className="mt-4 p-2 bg-green-500 text-white rounded-md">
              保存成功!
            </div>
          )}
        </div>

        {/* 中间错误信息 */}
        {error && (
          <div className="text-red-700 bg-red-100 dark:bg-red-200 dark:text-red-800 p-4 rounded-md pointer-events-auto">{error}</div>
        )}
      </div>
    </div>
  );
}

export default LiveScannerPage;
